"""Checks the market status."""
from datetime import datetime

from pydantic import BaseModel, Field

from stake.common import BaseClient, camelcase
from stake.constant import Url

__all__ = ["MarketStatus"]


class Status(BaseModel):
    change_at: str = Field(alias="change_at")
    next: str
    current: str

    class Config:
        alias_generator = camelcase


class MarketStatus(BaseModel):
    message: str
    unixtime: datetime
    error: str
    status: Status
    elapsedtime: int
    date: datetime
    version_number: str

    class Config:
        alias_generator = camelcase


class MarketClient(BaseClient):
    async def get(self) -> MarketStatus:
        data = await self._client.get(Url.market_status)
        return MarketStatus(**data["response"])

    async def is_open(self) -> bool:
        status = await self.get()
        return status.status.current == "open"



import weakref
from enum import Enum
from functools import partial

import inflection

camelcase = partial(inflection.camelize, uppercase_first_letter=False)


class SideEnum(str, Enum):
    BUY = "B"
    SELL = "S"


class BaseClient:
    # flake8: noqa
    def __init__(self, client: "stake._StakeClient"):  # type: ignore
        self._client = weakref.proxy(client)



import os
from typing import Optional, Union
from urllib.parse import urljoin

import aiohttp
from dotenv import load_dotenv
from pydantic import BaseModel, Field

from stake import (
    constant,
    equity,
    funding,
    fx,
    market,
    order,
    product,
    trade,
    transaction,
    user,
    watchlist,
)
from stake.common import camelcase

load_dotenv()

__all__ = ["StakeClient", "CredentialsLoginRequest", "SessionTokenLoginRequest"]

global test_name


class CredentialsLoginRequest(BaseModel):
    username: str = Field(default_factory=lambda *_: os.getenv("STAKE_USER", ""))
    password: str = Field(default_factory=lambda *_: os.getenv("STAKE_PASS", ""))
    otp: Optional[int] = None  # This is the code for 2FA
    remember_me_days: int = 30

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True


class SessionTokenLoginRequest(BaseModel):
    """Token based authentication, use this if 2FA is enabled."""

    token: str = Field(default_factory=lambda *_: os.getenv("STAKE_TOKEN", ""))


class Headers(BaseModel):
    accept: str = Field("application/json", alias="Accept")
    content_type: str = Field("application/json", alias="Content-Type")
    stake_session_token: Optional[str] = Field("", alias="Stake-Session-Token")


class HttpClient:
    """Handles http calls to the Stake API."""

    _session = aiohttp.ClientSession

    @staticmethod
    def url(endpoint: str) -> str:
        """Generates a stake api url.

        Args:
            endpoint (str): the final part of the url

        Returns:
            str: the full url
        """
        return urljoin(constant.STAKE_URL, endpoint, allow_fragments=True)

    @staticmethod
    async def get(url: str, payload: dict = None, headers: dict = None) -> dict:
        async with aiohttp.ClientSession(
            headers=headers, raise_for_status=True
        ) as session:
            response = await session.get(
                HttpClient.url(url), headers=headers, json=payload
            )
            return await response.json()

    @staticmethod
    async def post(url: str, payload: dict, headers: dict = None) -> dict:
        async with aiohttp.ClientSession(
            headers=headers, raise_for_status=True
        ) as session:
            response = await session.post(
                HttpClient.url(url), headers=headers, json=payload
            )
            return await response.json()

    @staticmethod
    async def delete(url: str, payload: dict = None, headers: dict = None) -> bool:
        async with aiohttp.ClientSession(
            headers=headers, raise_for_status=True
        ) as session:
            response = await session.delete(
                HttpClient.url(url), headers=headers, json=payload
            )
            return response.status <= 399


class InvalidLoginException(Exception):
    pass


class StakeClient:
    """The main client to interact with the Stake API."""

    def __init__(
        self, request: Union[CredentialsLoginRequest, SessionTokenLoginRequest] = None
    ):
        self.user: Optional[user.User] = None

        self.headers = Headers()
        self.httpClient = HttpClient

        # register all the clients
        self.equities = equity.EquitiesClient(self)
        self.fx = fx.FxClient(self)
        self.fundings = funding.FundingsClient(self)
        self.market = market.MarketClient(self)
        self.orders = order.OrdersClient(self)
        self.products = product.ProductsClient(self)
        self.trades = trade.TradesClient(self)
        self.transactions = transaction.TransactionsClient(self)
        self.watchlist = watchlist.WatchlistClient(self)

        self._login_request = request or SessionTokenLoginRequest()

    async def get(self, url: str, payload: dict = None) -> dict:
        """Performs an HTTP get operation.

        Args:
            url (str): the current endpoint
            payload (dict): The request's body.

        Returns:
            dict: the json response
        """
        return await self.httpClient.get(
            url, payload=payload, headers=self.headers.dict(by_alias=True)
        )

    async def post(self, url: str, payload: dict) -> dict:
        """Performs an HTTP post operation.

        Args:
            url (str): the current endpoint
            payload (dict): The request's body.

        Returns:
            dict: the json response
        """
        return await self.httpClient.post(
            url, payload=payload, headers=self.headers.dict(by_alias=True)
        )

    async def delete(self, url: str, payload: dict = None) -> bool:
        """Performs an HTTP delete operation.

        Args:
            url (str): the current endpoint
            payload (dict, optional): The request's body. Defaults to None.

        Returns:
            bool: True if the deletion was successful.
        """
        return await self.httpClient.delete(
            url, headers=self.headers.dict(by_alias=True), payload=payload
        )

    async def login(
        self, login_request: Union[CredentialsLoginRequest, SessionTokenLoginRequest]
    ) -> user.User:
        """Authenticates to the stake api.

        Args:
            login_request (Union[CredentialsLoginRequest, SessionTokenLoginRequest]):
            you can authenticate either with your credentials(username, password)
            or with an existing session token.

        Returns:
            user.User: the authenticated user.
        """
        if isinstance(login_request, CredentialsLoginRequest):
            try:
                data = await self.post(
                    constant.Url.create_session, payload=login_request.dict()
                )

                self.headers.stake_session_token = data["sessionKey"]
            except aiohttp.client_exceptions.ClientResponseError as error:
                raise InvalidLoginException("Invalid Login Credentials") from error
        else:
            self.headers.stake_session_token = login_request.token
        try:
            user_data = await self.get(constant.Url.user)
        except aiohttp.client_exceptions.ClientResponseError as error:
            raise InvalidLoginException("Invalid Session Token") from error

        self.user = user.User(**user_data)
        return self.user

    async def __aenter__(self):
        await self.login(self._login_request)
        return self

    async def __aexit__(self, exc_type, exc, tb):
        pass



from datetime import datetime
from typing import List, Union

from pydantic import BaseModel, Field

from stake.common import BaseClient, camelcase
from stake.constant import Url
from stake.product import Product

__all__ = ["AddToWatchlistRequest", "RemoveFromWatchlistRequest"]


class AddToWatchlistRequest(BaseModel):
    symbol: str
    watching: bool = True


class RemoveFromWatchlistRequest(BaseModel):
    symbol: str
    watching: bool = False


class WatchlistResponse(BaseModel):
    symbol: str
    watching: bool


class WatchlistProduct(BaseModel):
    id: str = Field(alias="productWatchlistID")
    watched_date: datetime
    product: Product

    class Config:
        alias_generator = camelcase


class WatchlistClient(BaseClient):
    async def _modify_watchlist(
        self, request: Union[AddToWatchlistRequest, RemoveFromWatchlistRequest]
    ) -> WatchlistResponse:
        """Adds/remove to the watchlist.

        Args:
            request (Union[AddToWatchlistRequest, RemoveFromWatchlistRequest]):
                Either an add orremove request.

        Returns:
            WatchlistResponse: The result of the watchlist modification
        """
        product = await self._client.products.get(request.symbol)

        assert product
        payload = {
            "instrumentID": str(product.id),
            "userID": str(self._client.user.id),
            "watching": request.watching,
        }
        data = await self._client.post(Url.watchlist_modify, payload=payload)

        return WatchlistResponse(symbol=request.symbol, watching=data["watching"])

    async def add(self, request: AddToWatchlistRequest) -> WatchlistResponse:
        """Adds a symbol to the watchlist.

        Args:
            request (AddToWatchlistRequest): The request containing the symbol.

        Returns:
            WatchlistResponse: The result of the watchlist modification.
        """
        return await self._modify_watchlist(request)

    async def remove(self, request: RemoveFromWatchlistRequest) -> WatchlistResponse:
        """Removes a symbol from the watchlist.

        Args:
            request (RemoveFromWatchlistRequest): The request containing the symbol

        Returns:
            WatchlistResponse: The result of the watchlist modification.
        """
        return await self._modify_watchlist(request)

    async def list(self) -> List[WatchlistProduct]:
        """Lists all the contents of your watchlist.

        Returns:
            List[WatchlistProduct]: The list of items in your watchlist.
        """
        watchlist = await self._client.get(
            Url.watchlist.format(userId=self._client.user.id)  # type: ignore
        )
        return [
            WatchlistProduct(**watched) for watched in watchlist["instrumentsWatchList"]
        ]



"""Currency conversion."""
from enum import Enum

from pydantic import BaseModel
from pydantic.types import UUID4

from stake.common import BaseClient, camelcase
from stake.constant import Url

__all__ = ["FxConversionRequest", "CurrencyEnum"]


class CurrencyEnum(str, Enum):
    AUD: str = "AUD"
    USD: str = "USD"


class FxConversionRequest(BaseModel):
    from_currency: CurrencyEnum
    to_currency: CurrencyEnum
    from_amount: float

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True


class FxConversion(BaseModel):
    from_currency: CurrencyEnum
    to_currency: CurrencyEnum
    from_amount: float
    to_amount: float
    rate: float
    quote: UUID4

    class Config:
        alias_generator = camelcase


class FxClient(BaseClient):
    async def convert(
        self, currency_conversion_request: FxConversionRequest
    ) -> FxConversion:
        """Converts from one currency to another."""
        data = await self._client.post(
            Url.rate, payload=currency_conversion_request.dict(by_alias=True)
        )
        return FxConversion(**data)



from datetime import datetime
from enum import IntEnum
from typing import List, Union

from pydantic import BaseModel, Field

from stake.common import BaseClient, SideEnum, camelcase
from stake.constant import Url

__all__ = ["OrderTypeEnum", "CancelOrderRequest"]


class OrderTypeEnum(IntEnum):
    MARKET = 1
    LIMIT = 2
    STOP = 3


class Order(BaseModel):
    order_no: str
    order_id: str = Field(alias="orderID")
    order_cash_amt: int
    symbol: str
    price: int
    stop_price: int
    side: SideEnum
    order_type: OrderTypeEnum
    cum_qty: str
    limit_price: int
    created_when: datetime
    order_status: int
    order_qty: float
    description: str
    instrument_id: str = Field(alias="instrumentID")
    image_url: str
    instrument_symbol: str
    instrument_name: str
    encoded_name: str

    class Config:
        alias_generator = camelcase


class CancelOrderRequest(BaseModel):
    order_id: str


class OrdersClient(BaseClient):
    """This client is in charge of dealing with your pending orders.

    These are the orders limit/stop etc.. that have not been traded yet.
    """

    async def list(self) -> List[Order]:
        """Lists all your pending orders.

        Returns:
            List[Order]: The list of pending orders.
        """
        data = await self._client.get(Url.orders)
        return [Order(**d) for d in data]

    async def cancel(self, order: Union[Order, CancelOrderRequest]) -> bool:
        """Cancels a pending order.

        Args:
            order (Union[Order, CancelOrderRequest])): an existing order or its ID.

        Returns:
            bool: True if the deletion was succesful.
        """
        return await self._client.delete(
            Url.cancel_order.format(orderId=order.order_id)  # type: ignore
        )



from typing import Optional

from pydantic import BaseModel
from pydantic.fields import Field

from stake.common import camelcase


class User(BaseModel):
    id: str = Field(alias="userId")
    first_name: str
    last_name: str
    email_address: str
    mac_status: str
    account_type: str
    region_identifier: str
    dw_account_number: Optional[str] = Field(alias="dw_AccountNumber")
    can_trade_on_unsettled_funds: Optional[bool]
    username: Optional[str]

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True



from typing import Any, List, Optional

from pydantic import BaseModel
from pydantic.fields import Field
from pydantic.types import UUID4

from stake.common import BaseClient, camelcase
from stake.constant import Url

__all__ = ["ProductSearchByName"]


class ProductSearchByName(BaseModel):
    """Request used to search for Products by their name or description."""

    keyword: str


class Instrument(BaseModel):
    encoded_name: str
    image_url: str
    instrument_id: str
    name: str
    symbol: str

    class Config:
        alias_generator = camelcase


class Product(BaseModel):
    id: UUID4
    instrument_type_id: Optional[str] = Field(None, alias="instrumentTypeID")
    symbol: str
    description: str
    category: str
    currency_id: Optional[str] = Field(None, alias="currencyID")
    url_image: str
    sector: Optional[str] = None
    parent_id: Optional[str] = Field(None, alias="parentID")
    name: str
    daily_return: float
    daily_return_percentage: float
    last_traded: float
    monthly_return: int
    yearly_return_percentage: float
    yearly_return_value: float
    popularity: int
    watched: int
    news: int
    bought: int
    viewed: int
    product_type: str
    trade_status: int
    encoded_name: str
    period: str
    inception_date: Optional[str]
    instrument_tags: List[Any]
    child_instruments: List[Instrument]

    class Config:
        alias_generator = camelcase


class ProductsClient(BaseClient):
    async def get(self, symbol: str) -> Optional[Product]:
        """Given a symbol it will return the matching product.

        Examples:
            tesla_product = self.get("TSLA")
        """
        data = await self._client.get(Url.symbol.format(symbol=symbol))  # type: ignore

        if not data["products"]:
            return None

        return Product(**data["products"][0])

    async def search(self, request: ProductSearchByName) -> List[Instrument]:
        products = await self._client.get(
            Url.products_suggestions.format(keyword=request.keyword)  # type: ignore
        )
        return [Instrument(**product) for product in products["instruments"]]

    async def product_from_instrument(
        self, instrument: Instrument
    ) -> Optional[Product]:
        return await self.get(instrument.symbol)



import re
from datetime import datetime
from enum import Enum
from typing import Optional, Union

from pydantic import BaseModel, Field, validator

from stake.common import BaseClient, camelcase
from stake.constant import Url

failed_transaction_regex = re.compile(r"^[0-9]{4}")

__all__ = [
    "LimitBuyRequest",
    "LimitSellRequest",
    "MarketBuyRequest",
    "MarketSellRequest",
    "StopBuyRequest",
    "StopSellRequest",
    "TradeType",
]


class TradeType(str, Enum):
    """The type of trade the user is requesting."""

    MARKET: str = "market"
    LIMIT: str = "limit"
    STOP: str = "stop"


class MarketBuyRequest(BaseModel):
    """This is the request that needs to be passed to the trade client in order
    to buy some equity."""

    # AAPL, MSFT, TSLA etc...
    symbol: str
    amount_cash: float
    comments: Optional[str]

    item_type: str = "instrument"
    order_type: TradeType = TradeType.MARKET

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True


class LimitBuyRequest(BaseModel):
    symbol: str
    limit_price: float
    quantity: int
    comments: Optional[str]

    item_type: str = "instrument"
    order_type: TradeType = TradeType.LIMIT

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True


class StopBuyRequest(BaseModel):
    symbol: str
    amount_cash: float
    price: float  # must be higher than the current one
    comments: Optional[str]

    item_type: str = "instrument"
    order_type: TradeType = TradeType.STOP

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True

    @validator("amount_cash")
    def at_least_10(cls, v: float) -> float:  # noqa

        if v < 10.0:
            raise ValueError("'amount_cash' must be at least '10$'.")

        return v


class LimitSellRequest(BaseModel):
    symbol: str
    limit_price: float
    quantity: int
    comments: Optional[str]

    item_type: str = "instrument"
    order_type: TradeType = TradeType.LIMIT

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True


class StopSellRequest(BaseModel):
    symbol: str
    quantity: float
    stop_price: float
    comments: Optional[str]

    item_type: str = "instrument"
    order_type: TradeType = TradeType.STOP

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True


class MarketSellRequest(BaseModel):
    """Sell at marked price."""

    symbol: str
    quantity: float
    comments: Optional[str]

    item_type: str = "instrument"
    order_type: TradeType = TradeType.MARKET

    class Config:
        alias_generator = camelcase
        allow_population_by_field_name = True


class TradeResponse(BaseModel):
    """The response from a request to buy/sell."""

    amount_cash: Optional[float]
    category: str
    commission: Optional[float]
    description: Optional[str]
    dw_order_id: str
    effective_price: Optional[float]
    encoded_name: str
    id: str
    image_url: str = Field(alias="imageURL")
    inserted_date: datetime
    item_id: str
    limit_price: Optional[float]
    name: str
    order_reject_reason: Optional[str]
    quantity: Optional[float]
    side: str
    status: Optional[int]
    stop_price: Optional[float]
    symbol: str
    updated_date: datetime

    class Config:
        alias_generator = camelcase


class TradesClient(BaseClient):
    """This client is used to buy/sell equities."""

    async def _trade(
        self,
        url: str,
        request: Union[
            MarketBuyRequest,
            LimitBuyRequest,
            StopBuyRequest,
            LimitSellRequest,
            StopSellRequest,
            MarketSellRequest,
        ],
        check_success: bool = True,
    ) -> TradeResponse:
        """A generic function used to submit a trade, either buy or sell.

        Args:
            url: the url for buy/sell
            request:
            check_success: if true, an extra check will be performed to see if
            the transaction really occurred. This should not be needed, since it should
            be possible to inspect the TradeResponse for errors, but it does not look
            like they are recorded anywhere.

        Returns:
            the TradeResponse

        Raises:
            RuntimeError
        """
        product = await self._client.products.get(request.symbol)
        assert product
        request_dict = request.dict(by_alias=True)
        request_dict["orderType"] = request_dict["orderType"].value
        request_dict["userId"] = str(self._client.user.id)
        request_dict["itemId"] = str(product.id)
        data = await self._client.post(url, request_dict)
        trade = TradeResponse(**data[0])

        if check_success:
            await self._verify_successful_trade(trade)

        return trade

    async def _verify_successful_trade(self, trade: TradeResponse) -> None:
        """We check the status of the trade by trying to find the matching
        transaction and inspecting its properties. This should not be needed
        but there is nothing i can see in the trading response that would help
        figuring out if the trade request was successful or not.

        Args:
            trade: the responded trade

        Raises:
            RuntimeError if the trade was not successful.
        """

        transactions = await self._client.get("users/accounts/transactions")

        if not transactions:
            raise RuntimeError(
                "The trade did not succeed (Reason: no transaction found)."
            )

        # wait for the transaction to be available
        for transaction in transactions:
            if transaction["orderId"] == trade.dw_order_id:
                if re.search(failed_transaction_regex, transaction["updatedReason"]):
                    raise RuntimeError(
                        "The trade did not succeed "
                        f"(Reason: {transaction['updatedReason']}"
                    )
                else:
                    return

        raise RuntimeError("Could not find a matching transaction.")

    async def buy(
        self, request: Union[MarketBuyRequest, LimitBuyRequest, StopBuyRequest]
    ) -> TradeResponse:
        """Creates an order to buy equities.

        Args:
            request: the buy request

        Returns:
            the TradeResponse object
        """
        return await self._trade(Url.quick_buy, request)

    async def sell(self, request: MarketSellRequest) -> TradeResponse:
        """Creates an order to sell equities.

        Args:
            request: the sell request

        Returns:
            the TradeResponse object
        """
        return await self._trade(Url.sell_orders, request)



from datetime import datetime, timedelta
from enum import Enum
from typing import List, Optional

from pydantic import BaseModel, Field
from pydantic.types import UUID4

from stake.common import BaseClient, camelcase
from stake.constant import Url

__all__ = ["TransactionRecordRequest"]


class TransactionRecordEnumDirection(str, Enum):
    prev: str = "prev"
    next_: str = "next"


class TransactionRecordRequest(BaseModel):
    to: datetime = Field(default_factory=datetime.utcnow)
    from_: datetime = Field(
        default_factory=lambda *_: datetime.utcnow() - timedelta(days=365)
    )
    limit: int = 100
    offset: Optional[int]
    direction: TransactionRecordEnumDirection = TransactionRecordEnumDirection.prev


class Instrument(BaseModel):
    id: UUID4
    symbol: str
    name: str


class Transaction(BaseModel):
    account_amount: float
    account_balance: float
    account_type: str
    comment: str
    dividend_tax: Optional[float] = None
    dividend: Optional[float] = None
    dnb: bool
    fee_base: int
    fee_exchange: int
    fee_sec: float
    fee_taf: float
    fee_xtra_shares: int
    fill_px: float
    fill_qty: float
    fin_tran_id: str = Field(alias="finTranID")
    fin_tran_type_id: str = Field(alias="finTranTypeID")
    instrument: Optional[Instrument] = None
    merger_acquisition: Optional[float] = None
    order_id: str = Field(alias="orderID")
    order_no: str
    position_delta: Optional[float] = None
    send_commission_to_inteliclear: bool
    symbol: Optional[str]
    system_amount: int
    tran_amount: float
    tran_source: str
    tran_when: datetime
    updated_reason: Optional[str]
    wlp_amount: int
    wlp_fin_tran_type_id: UUID4 = Field(None, alias="wlpFinTranTypeID")

    class Config:
        alias_generator = camelcase


class TransactionsClient(BaseClient):
    async def list(self, request: TransactionRecordRequest) -> List[Transaction]:
        """Returns the transactions executed by the user.

        Args:
            request (TransactionRecordRequest): specify the from+/to datetimes
              for the transaction collection.

        Returns:
            List[Transaction]: the transactions executed in the time frame.
        """
        payload = request.dict()
        payload.update(
            {"to": payload["to"].isoformat(), "from": payload.pop("from_").isoformat()}
        )

        data = await self._client.post(Url.account_transactions, payload=payload)
        transactions = []
        for d in data:
            if d.get("instrument"):
                d["symbol"] = d.get("instrument")["symbol"]
            transactions.append(Transaction(**d))
        return transactions



# flake8: noqa
from pathlib import Path

from single_version import get_version

from .client import *
from .funding import *
from .fx import *
from .market import *
from .order import *
from .product import *
from .trade import *
from .transaction import *
from .watchlist import *

__version__ = get_version("awesome_name", Path(__file__).parent.parent)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['stake']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.6.2,<4.0.0',
 'inflection>=0.5.0,<0.6.0',
 'pydantic>=1.5.1,<2.0.0',
 'python-dotenv>=0.13.0,<0.14.0',
 'single-version>=1.2.2,<2.0.0']

setup_kwargs = {
    'name': 'stake',
    'version': '0.1.6',
    'description': 'Unofficial https://stake.com.au API wrapper.',
    'long_description': '[![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)](<https://github.com/pre-commit/pre-commit>)\n![Coverage](coverage.svg)\n\n# Stake\n\n**Stake** is an unofficial Python client for the [Stake](<https://www.stake.com.au>) trading platform.\n\nThis library wraps the current Stake RPC api and allows common trade operations, such as submitting buy/sell requests, checking your portfolio etc...\n\nPlease note that, at the current stage, the Stake client is asynchronous.\n\n## Installation\n\n~~~$\npip install stake\n~~~\n\n## Quickstart\n\nAfter you install the package, you will need to authenticate to Stake in order to get authorization to interact with your account.\nIn order to successfully issue requests to the Stake platform you will need to authenticate to it. Every requests to the Stake endpoints will need to contain a `Stake-Session-Token` in the request headers.\n\n## Using an existing Session-Token\n\nYou can retrieve one of these `Stake-Session-Token` by using the developer tools in your browser for example and inspecting some of the request headers sent to some of the `https://global-prd-api.hellostake.com/` host.\n\nThey are usually valid for 30 days and seem to get refreshed before expiry so you should be good to use them directly.\n\nIf you already have an existing token you can pass it on to the `StakeClient` as such:\n\n~~~python\n\nfrom stake import StakeClient, SessionTokenLoginRequest, CredentialsLoginRequest\nimport asyncio\n\nlogin_request = SessionTokenLoginRequest(token="secrettoken")\nasync def print_user():\n    async with StakeClient(login_request) as stake_session:\n        print(stake_session.user.first_name)\n        print(stake_session.headers.stake_session_token)\n\nasyncio.run(print_user())\n~~~\n\n> **_NOTE:_**  The default value of the token is read from the `STAKE_TOKEN` environment variable. If you have that env-var set you should be able to just use:\n> `async with StakeClient() as stake_session: ...`\n\n## Login with your credentials\n\nIf you prefer to pass in your username/password credentials to login instead, it\'s easy to do:\n\n### If you do not have two-factor authentication enabled:\n\n~~~python\n\nfrom stake import StakeClient, SessionTokenLoginRequest, CredentialsLoginRequest\nimport asyncio\n\nlogin_request = CredentialsLoginRequest(\n    username="youruser@name.com", # os.getenv("STAKE_USER") by default\n    password="yoursecretpassword") # os.getenv("STAKE_PASS") by default\n\nasync def print_user():\n    async with StakeClient(login_request) as stake_session:\n        print(stake_session.user.first_name)\n        print(stake_session.headers.stake_session_token)\n\nasyncio.run(print_user())\n~~~\n\n### If you have two-factor authentication enabled:\n\nIn this case you need to have your phone around, get the current code from the authenticator app and add it to the `CredentialsLoginRequest` as such:\n\n~~~python\n    login_request = CredentialsLoginRequest(username="youruser@name.com",password="yoursecretpassword",\n        otp="Your-authenticator-app-code")\n~~~\n\nObviously, this can become a bit inconvenient, since you will need to provide the otp code every time you instantiate a new `StakeClient` instance. Therefore, you could probably authenticate once with your credentials, retrieve the session token from the headers(`stake_session.headers.stake_session_token`), and store it in the `STAKE_TOKEN` env-var for subsequent usages.\n\n## Examples\n\nWith `stake-python` you can do most of the operations that are available through the web app.\n\nHere are some examples:\n\n### Display the contents of your portfolio\n\n~~~python\nfrom stake import StakeClient, SessionTokenLoginRequest, CredentialsLoginRequest\nimport asyncio\n\n\nasync def show_portfolio():\n    # here the client will use the STAKE_TOKEN env var for authenticating\n    async with StakeClient() as stake_session:\n        my_equities = await stake_session.equities.list()\n\n        for my_equity in my_equities.equity_positions:\n            print(my_equity.symbol, my_equity.yearly_return_value)\n\n        return my_equities\n\nasyncio.run(show_portfolio())\n~~~\n\nWhich will return something like:\n\n~~~\nAAPL 80.48\nADBE 251.35\nGOOG 559.89\nGRPN -13.77\nHTZ -10.52\nMSFT 97.14\nNFLX 263.55\nNIO 17.3\nNVDA 410.04\nOKTA 96.31\nSHOP 690.68\nSPOT 142.88\nSQ 101.75\nTQQQ 115.82\nTSLA 402.37\nVGT 130.08\nZM 331.1\n~~~\n\n### Buy/Sell shares\n\nYou can send buy/sell orders to the platform quite easily by just issuing trade requests.\nPlease check the `stake.trade` module for more details.\n\n~~~python\n\nasync def example_limit_buy():\n    symbol = "UNKN" # should be the equity symbol, for ex: AAPL, TSLA, GOOGL\n    async with StakeClient() as stake_session:\n        return trades.buy(\n            LimitBuyRequest(symbol=symbol, limitPrice=10, quantity=1000)\n        )\n\nasyncio.run(example_limit_buy())\n~~~\n\nTo perform multiple requests at once you can use an `asyncio.gather` operation to run all the buy trades in parallel.\n\n~~~python\n\nasync def example_stop_sell():\n    """THis example will add a stop sell request for one of your equities"""\n    my_equities = await show_portfolio()\n\n    symbol = "T.SLA" # mispelt on purpose so that no trade actually happens, should be TSLA.\n    tsla_equity = [equity for equity in my_equities.equity_positions if equity.symbol == symbol]\n\n\n    stop_sell_request = stake.StopSellRequest(symbol=tsla_equity.symbol,\n                                                  stopPrice=stop_price,\n                                                  comment="My stop sell.",\n                                                  quantity=current_equity.available_for_trading_qty)\n        result = await stake_client.trades.sell(request=stop_sell_request)\n    symbol = "UNKN" # should be the equity symbol, for ex: AAPL, TSLA, GOOGL\n    async with StakeClient() as stake_session:\n        return trades.buy(\n            LimitBuyRequest(symbol=symbol, limitPrice=10, quantity=1000)\n        )\n\nasyncio.run(example_limit_buy())\n~~~\n\n## Contributors\n\n### Contributors on GitHub\n\n* [Contributors](<https://github.com/stabacco/stake-python/graphs/contributors>)\n\n## License\n\n* see [LICENSE](<https://github.com/stabacco/stake-python/blob/master/LICENSE.md>) file\n\n## Contact\n\n* Created by [Stefano Tabacco](<https://github.com/stabacco>)\n',
    'author': 'Stefano Tabacco',
    'author_email': 'tabacco.stefano@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/stabacco/stake-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from enum import Enum
from typing import List, Optional

from pydantic import UUID4, BaseModel, Field

from stake.common import BaseClient, SideEnum, camelcase
from stake.constant import Url


class EquityCategory(str, Enum):
    ETF = "ETF"
    STOCK = "Stock"


class EquityPosition(BaseModel):
    available_for_trading_qty: float
    average_price: float = Field(alias="avgPrice")
    category: EquityCategory
    cost_basis: float
    daily_return_value: float
    encoded_name: str
    instrument_id: UUID4 = Field(alias="instrumentID")
    last_trade: float
    market_value: float
    market_price: float = Field(alias="mktPrice")
    name: str
    open_qty: float
    period: str
    prior_close: float
    side: SideEnum
    symbol: str
    unrealized_day_pl: float = Field(alias="unrealizedDayPL")
    unrealized_day_pl_percent: float = Field(alias="unrealizedDayPLPercent")
    unrealized_pl: float = Field(alias="unrealizedPL")
    url_image: str
    yearly_return_percentage: float
    yearly_return_value: float
    ask_price: Optional[float] = None
    bid_price: Optional[float] = None
    return_on_stock: Optional[float] = None

    class Config:
        alias_generator = camelcase


class EquityPositions(BaseModel):
    """Represents the user's portforlio, with the list of the currently
    available equities."""

    equity_positions: List[EquityPosition]
    equity_value: float
    prices_only: bool

    class Config:
        alias_generator = camelcase


class EquitiesClient(BaseClient):
    async def list(self) -> EquityPositions:
        """Displays the contents of your portfolio.

        Returns:
            EquityPositions: The list of your equities.
        """
        data = await self._client.get(Url.equity_positions)
        return EquityPositions(**data)



from enum import Enum


class Url(str, Enum):
    """Contains all the visited stake urls."""

    account_balance: str = "cma/getAccountBalance"
    account_transactions: str = "users/accounts/accountTransactions"
    cash_available: str = "users/accounts/cashAvailableForWithdrawal"
    create_session: str = "sessions/createSession"
    equity_positions: str = "users/accounts/equityPositions"
    fund_details: str = "fund/details"
    fundings: str = "utils/activityLog/fundingOnly"
    market_status: str = "utils/marketStatus"
    orders: str = "users/accounts/v2/orders"
    quotes: str = "quotes/marketData/{symbols}"
    rate: str = "wallet/rate"
    transactions: str = "users/accounts/transactions"
    user: str = "user"
    quick_buy: str = "purchaseorders/v2/quickBuy"
    sell_orders: str = "sellorders"
    cancel_order: str = "orders/cancelOrder/{orderId}"
    symbol: str = "products/searchProduct?symbol={symbol}&page=1&max=1"
    products_suggestions: str = "products/getProductSuggestions/{keyword}"
    watchlist_modify: str = "instruments/addRemoveInstrumentWatchlist"
    watchlist: str = "products/productsWatchlist/{userId}"


STAKE_URL = "https://global-prd-api.hellostake.com/api/"



"""Your current fundings."""
from datetime import date, datetime, timedelta
from typing import List, Optional

from pydantic import BaseModel, Field
from pydantic.types import UUID4

from stake.common import BaseClient, camelcase
from stake.constant import Url

__all__ = ["FundingRequest"]


class FundingRequest(BaseModel):
    """Request to be issued to the fundings endpoint."""

    end_date: date = Field(default_factory=date.today)
    start_date: date = Field(
        default_factory=lambda *_: date.today() - timedelta(days=365)
    )


class Funding(BaseModel):
    id: UUID4
    timestamp: datetime
    order_type: str
    event_type: str
    status: str
    title: str
    amount: str
    description: str
    currency_from: str
    currency_to: str
    spot_rate: float
    total_fee: float
    amount_from: float
    amount_to: float
    rate: float
    reference_number: str

    class Config:
        alias_generator = camelcase


class CashSettlement(BaseModel):
    utc_time: datetime
    cash: float

    class Config:
        alias_generator = camelcase


class CashAvailable(BaseModel):
    cash_available_for_withdrawal: float
    cash_available_for_trade: float
    cash_balance: float
    reserved_cash: float
    dw_cash_available_for_withdrawal: float
    pending_orders_amount: float
    pending_withdrawals: float
    card_hold_amount: float
    pending_poli_amount: float
    cash_settlement: List[Optional[CashSettlement]]

    class Config:
        alias_generator = camelcase


class FundingsClient(BaseClient):
    async def list(self, request: FundingRequest) -> List[Funding]:
        payload = {
            "endDate": request.end_date.strftime("%d/%m/%Y"),
            "startDate": request.start_date.strftime("%d/%m/%Y"),
        }
        data = await self._client.post(Url.fundings, payload=payload)

        return [Funding(**d) for d in data]

    async def in_flight(self) -> dict:
        """Returns the funds currently in flight."""
        return await self._client.get(Url.fund_details)["fundsInFlight"]

    async def cash_available(self) -> CashAvailable:
        data = await self._client.get(Url.cash_available)
        return CashAvailable(**data)
